-- DarkSlayerEX's Item Tweaker Core: an API for tweaking existing items without redefining them entirely.
--Initializes the tables needed for the code to run
if not ItemTweaker then  ItemTweaker = {} end
if not TweakItem then  TweakItem = {} end
if not TweakItemData then  TweakItemData = {} end
if not TweakItemBackupValue then  TweakItemBackupValue = {} end
if not ItemTweaker.ItemTweaker_applied then ItemTweaker.ItemTweaker_applied = false end

function ItemTweaker.callIfCallable(f)
    return function(...)
        error, result = pcall(f, ...)
        if error then -- f exists and is callable
            print('ok')
            return result
        end
        -- nothing to do, as though not called, or print('error', result)
    end
end

function ItemTweaker.printJavaArray(javaArray)
    for index=0, javaArray:size()-1 do
        print(index, javaArray:get(index))
    end
end

function ItemTweaker.convertArrayToString(array,sep)
    --ItemTweaker.printJavaArray(array);
    --return table.concat(array,sep);
    
    local arraySize = array:size();
    local result = "";
    if arraySize > 0 then
        result = tostring(array:get(0));
    end
    
    for index=1, arraySize-1 do
        --print("ItemTweaker.convertArrayToString[".. index.. "] " .. result .. " + " .. sep);
        result = result .. sep .. tostring(array:get(index));
    end
    return result;
end

local categoriesofitemwithoutcategories = nil;
function getEmptyJavaArrayString()
    if categoriesofitemwithoutcategories == nil then
        categoriesofitemwithoutcategories = ScriptManager.instance:getItem("ItemTweaker.ItemTweakerDefaultItem"):getCategories();--use an object unlikely to become a weapon soon
    end
    return categoriesofitemwithoutcategories;
end

--Prep code to make the changes to all item in the TweakItemData table.
function ItemTweaker.tweakItems()
    local Verbose = ItemTweaker.OPTIONS.Verbose;
    if Verbose == true then
        print("ItemTweaker.tweakItems")
    end
    local item;
    for itemName,propertyTable in pairs(TweakItemData) do 
        for property,value in pairs(propertyTable) do 
            item = ScriptManager.instance:getItem(itemName);
            if item ~= nil then
                if not TweakItemBackupValue[itemName] then--create backup for BetterSorting
                    TweakItemBackupValue[itemName] = {};
                    TweakItemBackupValue[itemName][itemPropertyLower] = nil
                end
                if TweakItemBackupValue[itemName][property] == nil then
                    TweakItemBackupValue[itemName][property] = getItemPropertyValue(item, property); -- does this even work ?
                end
                if (ItemTweaker.ItemTweaker_applied == false and TweakItemBackupValue[itemName][property] ~= nil) then
                    ItemTweaker.ItemTweaker_applied = true;
                    if Verbose == true then
                        print("ItemTweaker next main menu should do rollback.");
                    end
                end
                if Verbose == true then
                    if TweakItemBackupValue[itemName][property] == nil then
                        print(itemName..": "..property..", ? => "..value); -- log what we intend to do
                    else
                        print(itemName..": "..property..", "..TweakItemBackupValue[itemName][property].." => "..value); -- log what we intend to do
                    end
                end
                if (property=="categories") then
                    item:setCategories(getEmptyJavaArrayString());--DoParam does not clear categories array before adding so we need to call setCategories with empty array first
                end
                item:DoParam(property.." = "..value); -- do it
            end
        end
    end
end

--Prep code to remove SOME OF the changes applied on Items when goind back to main menu (allows to use save reload with mod swaps and intuitive items changes).
function ItemTweaker.rollbackItems()
    if ItemTweaker.ItemTweaker_applied == true and ItemTweaker.OPTIONS.Rollback == true then
        local Verbose = ItemTweaker.OPTIONS.Verbose;
        if Verbose == true then
            print("ItemTweaker.rollbackItems");
        end
        local item;
        for itemName,propertyTable in pairs(TweakItemBackupValue) do 
            for property,value in pairs(propertyTable) do
                if value ~= nil then
                    item = ScriptManager.instance:getItem(itemName);
                    if item ~= nil then
                        local modifiedValue = getItemPropertyValue(item, property);
                        if Verbose == true then
                            print(itemName..": "..property..", "..modifiedValue.." => "..value); -- log what we intend to do
                        end
                        if (property=="categories") then
                            item:setCategories(getEmptyJavaArrayString());--DoParam does not clear categories array before adding so we need to call setCategories with empty array first
                        end
                        item:DoParam(property.." = "..value); -- do it
                    end
                end
            end
        end
        
        ItemTweaker.ItemTweaker_applied = false
    end
end

function TweakItem(itemName, itemProperty, propertyValue)
    local itemPropertyLower = string.lower(itemProperty);
    if not TweakItemData[itemName] then
        TweakItemData[itemName] = {};
    end
    if not TweakItemBackupValue[itemName] then
        TweakItemBackupValue[itemName] = {};
    end
    TweakItemBackupValue[itemName][itemPropertyLower] = nil
    TweakItemData[itemName][itemPropertyLower] = propertyValue;
end

Events.OnGameBoot.Add(ItemTweaker.tweakItems)
Events.OnLoad.Add(ItemTweaker.tweakItems)
Events.OnPostSave.Add(ItemTweaker.rollbackItems)
--[[
-------------------------------------------------
--------------------IMPORTANT--------------------
-------------------------------------------------
You should be able to modify any aspect of an item such as: DisplayName, DisplayCategory, Weight, Icon, StressReduction, HungerChange
        HOWEVER
DO NOT MODIFY AN ITEM'S TYPE UNLESS YOU WANT PLAYERS TO START A NEW WORLD. ITEMS WITH THEIR TYPES CHANGED ARE DELETED.
If you have a workaround for this in place however, then it should be okay.

You can make compatibility patches, allowing tweaks to only be applied under the proper circumstances.
Examples:


        TweakItemData["MyMod.MyItem"] = { ["DisplayCategory"] = "Weapon" };
        
        if getActivatedMods():contains("CustomCategories") then 
            TweakItemData["MyMod.MyItem"] = { ["DisplayCategory"] = "BluntWeapon" };
        end
        
and
        
        TweakItemData["MyMod.Book"] = { ["Weight"] = "0.8" };
        
        if getActivatedMods():contains("WeightlessBooks") then 
            TweakItemData["MyMod.Book"] = { ["Weight"] = "0" };
        end

]]
